<?php
namespace MascotCoreEdulerns\Widgets\Tutor;

use Elementor\Widget_Base;
use Elementor\Controls_Manager;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * Elementor Hello World
 *
 * Elementor widget for hello world.
 *
 * @since 1.0.0
 */
class TM_Elementor_Tutor_Courses extends Widget_Base {
	public function __construct($data = [], $args = null) {
		parent::__construct($data, $args);
		$direction_suffix = is_rtl() ? '.rtl' : '';

		wp_register_style( 'tm-tutor-courses-loader-style', MASCOT_CORE_EDULERNS_ASSETS_URI . '/css/shortcodes/tutor-courses-loader' . $direction_suffix . '.css' );
	}

	/**
	 * Retrieve the widget name.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return string Widget name.
	 */
	public function get_name() {
		return 'tm-ele-tutor-courses';
	}

	/**
	 * Retrieve the widget title.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return string Widget title.
	 */
	public function get_title() {
		return esc_html__( 'Tutor Courses Grid/List', 'mascot-core-edulerns' );
	}

	/**
	 * Retrieve the widget icon.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return string Widget icon.
	 */
	public function get_icon() {
		return 'tm-elementor-widget-icon';
	}

	/**
	 * Retrieve the list of categories the widget belongs to.
	 *
	 * Used to determine where to display the widget in the editor.
	 *
	 * Note that currently Elementor supports only one category.
	 * When multiple categories passed, Elementor uses the first one.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return array Widget categories.
	 */
	public function get_categories() {
		return [ 'tm' ];
	}

	/**
	 * Retrieve the list of scripts the widget depended on.
	 *
	 * Used to set scripts dependencies required to run the widget.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return array Widget scripts dependencies.
	 */
	public function get_script_depends() {
		return [ 'mascot-core-hellojs' ];
	}


	/**
	 * Get style dependencies.
	 *
	 * Retrieve the list of style dependencies the element requires.
	 *
	 * @since 1.9.0
	 * @access public
	 *
	 * @return array Element styles dependencies.
	 */
	public function get_style_depends() {
		return [ 'tm-tutor-courses-loader-style' ];
	}

	public function get_instance_value_skin( $key ) {
		$settings = $this->get_settings_for_display();

		if( !empty( $settings['_skin'] ) && isset( $settings[str_replace( '-', '_', $settings['_skin'] ) . '_' . $key] ) ) {
			 return $settings[str_replace( '-', '_', $settings['_skin'] ) . '_' . $key];
		}
		return $settings[$key];
	}

	public function get_course_categories() {
		if ( function_exists( 'tutor_utils' ) ) {
			$course_terms = get_terms( [
				'taxonomy'   => 'course-category',
				'parent'     => 0,
				'hide_empty' => '',
			] );
			$term_loop    = [];
			foreach ( $course_terms as $i => $course_term ) {
				$term_loop[ $course_term->slug ] = $course_term->name;
			}

			return $term_loop;
		}
	}

	protected function get_supported_ids() {
		$supported_ids = [];

		$wp_query = new \WP_Query( array(
			'post_type' => 'courses',
			'post_status' => 'publish'
		) );

		if ( $wp_query->have_posts() ) {
			while ( $wp_query->have_posts() ) {
				$wp_query->the_post();
				$supported_ids[get_the_ID()] = get_the_title();
			}
		}

		return $supported_ids;
	}


	/**
	 * Register the widget controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function register_controls() {
		$orderby_parameters_list1 = mascot_core_edulerns_orderby_parameters_list();
		$orderby_parameters_list2 = array(
		);
		$orderby_parameters_list = array_merge( $orderby_parameters_list2, $orderby_parameters_list1 );
		$this->start_controls_section(
			'general',
			[
				'label' => esc_html__( 'General', 'mascot-core-edulerns' ),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);
		$this->add_control(
			'display_type', [
				'label' => esc_html__( "Display Type", 'mascot-core-edulerns' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => [
					'grid'	=>	esc_html__( 'Grid', 'mascot-core-edulerns' ),
					'masonry'	=>	esc_html__( 'Masonry', 'mascot-core-edulerns' ),
					'carousel'	=>	esc_html__( 'Carousel', 'mascot-core-edulerns' ),
				],
				'default' => 'grid',
			]
		);
		$this->add_control(
			'columns', [
				'label' => esc_html__( "Columns Layout", 'mascot-core-edulerns' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => [
					'1'  =>  '1',
					'2'  =>  '2',
					'3'  =>  '3',
					'4'  =>  '4',
					'5'  =>  '5',
					'6'  =>  '6',
				],
				'default' => '3',
				'condition' => [
					'display_type!' => array('carousel')
				]
			]
		);

		//responsive grid layout
		mascot_core_edulerns_elementor_grid_responsive_columns($this);

		$this->add_control(
			'gutter',
			[
				'label' => esc_html__( "Gutter", 'mascot-core-edulerns' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => mascot_core_edulerns_isotope_gutter_list_elementor(),
				'default' => 'gutter-15',
			]
		);
		$this->end_controls_section();





		//Swiper Slider Options
		mascot_core_edulerns_get_swiper_slider_arraylist( $this, 1, '', array('display_type' => array('carousel') ) );
		mascot_core_edulerns_get_swiper_slider_nav_arraylist( $this, 1, '', array('display_type' => array('carousel') ) );
		mascot_core_edulerns_get_swiper_slider_dots_arraylist( $this, 1, '', array('display_type' => array('carousel') ) );












		$this->start_controls_section(
			'query', [
				'label' => esc_html__( 'Query', 'mascot-core-edulerns' ),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);
		$this->add_control(
			'total_items', [
				'label' => esc_html__( "Number of Items to Query from Database", 'mascot-core-edulerns' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				"description" => esc_html__( "How many items do you wish to show? Put -1 to show all. Default 3", 'mascot-core-edulerns' ),
				'default' => '3'
			]
		);
		$this->add_control(
			'ids',
			[
				'label' => __( 'Ids', 'mascot-core-edulerns' ),
				'type' => Controls_Manager::SELECT2,
				'options' => $this->get_supported_ids(),
				'label_block' => true,
				'multiple' => true,
			]
		);
		$this->add_control(
			'category',
			[
				'label' => __( 'Category', 'mascot-core-edulerns' ),
				'type' => Controls_Manager::SELECT2,
				'options' => $this->get_course_categories(),
				'label_block' => true,
				'multiple' => true,
			]
		);
		$this->add_control(
			'order_by', [
				'label' => esc_html__( "Order By", 'mascot-core-edulerns' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => $orderby_parameters_list,
			]
		);
		$this->add_control(
			'order', [
				'label' => esc_html__( "Order", 'mascot-core-edulerns' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => [
					'DESC' => esc_html__( 'Descending', 'mascot-core-edulerns' ),
					'ASC' => esc_html__( 'Ascending', 'mascot-core-edulerns' ),
				],
			]
		);
		$this->end_controls_section();










		//Content Options
		$this->start_controls_section(
			'content',
			[
				'label' => esc_html__( 'Contents Show/Hide', 'mascot-core-edulerns' ),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);
		$this->add_control(
			'show_thumb', [
				'label' => esc_html__( "Show Thumbnail", 'mascot-core-edulerns' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'default' => 'yes',
			]
		);
		$this->add_control(
			'feature_thumb_image_size', [
				'label' => esc_html__( "Thumbnail Image Size", 'mascot-core-edulerns' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => mascot_core_edulerns_get_available_image_sizes(),
				'default' => 'post-thumbnail',
				'condition' => [
					'show_thumb' => array('yes'),
				]
			]
		);
		$this->add_control(
			'show_title',
			[
				'label' => esc_html__( "Show Title", 'mascot-core-edulerns' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'default' => 'yes'
			]
		);
		$this->add_control(
			'title_tag',
			[
				'label' => esc_html__( "Title Tag", 'mascot-core-edulerns' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => mascot_core_edulerns_heading_tag_list(),
				'default' => 'h4',
				'condition' => [
					'show_title' => array('yes')
				]
			]
		);
		$this->add_control(
			'show_excerpt',
			[
				'label' => esc_html__( "Show Excerpt", 'mascot-core-edulerns' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'default' => 'yes'
			]
		);
		$this->add_control(
			'excerpt_length', [
				'label' => esc_html__( "Excerpt Length", 'mascot-core-edulerns' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				"description" => esc_html__( "Number of words to display. Example: 25. Default all", 'mascot-core-edulerns' ),
				'condition' => [
					'show_excerpt' => array('yes')
				]
			]
		);
		$this->add_control(
			'show_meta',
			[
				'label' => esc_html__( "Show Post Meta", 'mascot-core-edulerns' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'default' => 'yes'
			]
		);
		$this->add_control(
			'meta_options',
			[
				'label' => esc_html__( "Choose Meta", 'mascot-core-edulerns' ),
				'type' => \Elementor\Controls_Manager::SELECT2,
				'options' => [
					'show-date-over-image'	=>	esc_html__( 'Show Date Over Image', 'mascot-core-edulerns' ),
					'show-full-date'	=>	esc_html__( 'Show Full Date & Time', 'mascot-core-edulerns' ),
					'show-time'	=>	esc_html__( 'Show Only Time', 'mascot-core-edulerns' ),
					'show-venue'	=>	esc_html__( 'Show Venue', 'mascot-core-edulerns' ),
					'show-organizer'	=>	esc_html__( 'Show Organizer', 'mascot-core-edulerns' ),
					'show-cat'	=>	esc_html__( 'Show Category', 'mascot-core-edulerns' ),
				],
				'multiple' => true,
				'description' => esc_html__('Enable/Disabling this option will show/hide each Post Meta', 'mascot-core-edulerns'),
				'default' => [ 'show-date-over-image', 'show-time' ],
				'condition' => [
					'show_meta' => array('yes')
				]
			]
		);
		$this->add_control(
			'show_single_date_meta',
			[
				'label' => esc_html__( "Show Single Date Block", 'mascot-core-edulerns' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'default' => 'yes'
			]
		);
		$this->add_control(
			'show_single_date_meta_options',
			[
				'label' => esc_html__( "Choose Meta", 'mascot-core-edulerns' ),
				'type' => \Elementor\Controls_Manager::SELECT2,
				'options' => [
					'show-date'	=>	esc_html__( 'Show Only Date', 'mascot-core-edulerns' ),
					'show-time'	=>	esc_html__( 'Show Only Time', 'mascot-core-edulerns' ),
					'show-full-date'	=>	esc_html__( 'Show Full Date & Time', 'mascot-core-edulerns' ),
				],
				'default' => 'show-date',
				'condition' => [
					'show_single_date_meta' => array('yes')
				]
			]
		);
		$this->end_controls_section();





		//Category Filter
		$this->start_controls_section(
			'cat_filter_section', [
				'label' => esc_html__( 'Category Filter', 'mascot-core-edulerns' ),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);
		mascot_core_edulerns_get_cat_filter_arraylist( $this, 1, array('display_type' => array('grid', 'masonry', 'carousel') ) );
		mascot_core_edulerns_get_cat_filter_arraylist( $this, 2 );
		mascot_core_edulerns_get_cat_filter_arraylist( $this, 3 );
		mascot_core_edulerns_get_cat_filter_arraylist( $this, 4 );

		$this->end_controls_section();


		//add button controns
		mascot_core_edulerns_get_button_control($this, true);

		//add Loadmore button controns
		//mascot_core_edulerns_get_loadmore_button_control($this);
	}

	public function query_posts() {
		$settings = $this->get_settings_for_display();
		$paged = isset($settings['paged']) ? $settings['paged'] : '';

		//query args
		$args = array(
			'post_type' => 'courses',
			'orderby' => $settings['order_by'],
			'order' => $settings['order'],
			'posts_per_page' => $settings['total_items'],
			'paged' => $paged,
		);

		$meta_query = array();



		if( ! empty( $meta_query ) ) {
			$meta_query['relation'] = 'AND';
			$args['meta_query'] = $meta_query;
		}

		if( ! empty( $settings['ids'] ) ) {
			$args['post__in'] = $settings['ids'];
		}

		if( ! empty( $settings['category'] ) ) {
			$args['tax_query'] = array(
				array(
					'taxonomy' => 'course-category',
					'field'    => 'term_id',
					'terms'    => $settings['category'],
				),
			);
		}
		return $the_query = new \WP_Query( $args );
	}

	/**
	 * Render the widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function render() {
		$settings = $this->get_settings_for_display();
		$class_instance =  '';

		$settings['holder_id'] = mascot_core_edulerns_get_isotope_holder_ID('tutor');

		$this->render_output( $class_instance, $settings );
	}

	public function render_output( $class_instance, $settings ) {
		$new_cpt_class = $class_instance;
		$settings['the_query'] = $this->query_posts();



		//classes
		$classes = array();
		if( isset($settings['_skin']) ) {
			$classes[] = 'tutor-courses-' . $settings['_skin'];
		}

		$settings['classes'] = $classes;

		//button classes
		$settings['btn_classes'] = mascot_core_edulerns_prepare_button_classes_from_params( $settings );


		//event-date classes
		$event_left_classes = array();
		$settings['event_left_classes'] = $event_left_classes;

		$settings['settings'] = $settings;

		//Produce HTML version by using the parameters (filename, variation, folder name, parameters, shortcode_ob_start)
		$html = mascot_core_edulerns_get_shortcode_template_part( 'tutor-courses', $settings['display_type'], 'tutor-courses/tpl', $settings, true );

		echo $html;
	}
}